/*
Gamebots UT Copyright (c) 2002, Andrew N. Marshal, Gal Kaminka
Gamebots Pogamut derivation Copyright (c) 2010, Michal Bida

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This software must also be in compliance with the Epic Games Inc. license for mods which states the following: "Your mods must be distributed solely for free, period. Neither you, nor any other person or party, may sell them to anyone, commercially exploit them in any way, or charge anyone for receiving or using them without prior written consent of Epic Games Inc. You may exchange them at no charge among other end-users and distribute them to others over the Internet, on magazine cover disks, or otherwise for free." Please see http://www.epicgames.com/ut2k4_eula.html for more information.

*/

//=============================================================================
// BotConnection.
// Based connection class
//=============================================================================
class BotConnection extends GBClientClass;

//------------Variables---------------------------

// delay between visionUpdates
var config float visionTime;

// on / off all synchronous messages
var config bool bSynchronousMessagesOff;

var RemoteBot theBot;

var Actor tempActor;

var Actor FocusActor;

//switches for exporting information after READY command
var config bool bExportGameInfo;
var config bool bExportMutators;
var config bool bExportITC;
var config bool bExportNavPoints;
var config bool bExportMovers;
var config bool bExportInventory;
var config bool bExportPlayers;

//------------Events---------------------------

// triggered when a socket connection with the server is established
event Accepted()
{
	if(bDebug)
		`log("Accepted BotConnection" @ self);

	//TODO HACK
  visionTime = 0.25;
  
	SendLine("HELLO_BOT");


	if (BotDeathMatch(WorldInfo.Game).bPasswordProtected)
		gotoState('checkingPassword','Waiting');
	else
		gotoState('waiting','Waiting');
}
event Closed()
{
	Destroy();
}

event Destroyed()
{
	if ( theBot != None && theBot.Pawn != None )
	{
		theBot.SetLocation(theBot.Pawn.Location);
		theBot.Pawn.RemoteRole = ROLE_SimulatedProxy;
		theBot.Pawn.UnPossessed();
		theBot.Pawn.Destroy();
	}

	if (theBot != None)
	{
		theBot.Destroy();
	}

	if (FocusActor != None)
	{
		FocusActor.Destroy();
	}

}


//------------Functions---------------------------

//Init recieved from client
function InitBot()
{
	local string clientName, className, temp, DesiredSkin, outstring;
	local int teamNum;
	local vector StartLocation;
	local rotator StartRotation;
	local float DesiredSkill, DesiredAccuracy;
	local bool ShouldLeadTarget;

	clientName = GetArgVal("Name");

	DesiredSkin = GetArgVal("Skin");

	className = GetArgVal("ClassName");

	temp = GetArgVal("Team");
	if( temp != "" )
		teamNum = int(temp);
	else
		teamNum = 255;

	temp = GetArgVal("DesiredSkill");
	if (temp != "")
		DesiredSkill = float(temp);
	else
		DesiredSkill = -1; //that means deufault value will be used

	temp = GetArgVal("ShouldLeadTarget");
	if (temp != "")
		ShouldLeadTarget = bool(temp);
	else
		ShouldLeadTarget = false;

	// add the bot into the game
	theBot = BotDeathMatch(WorldInfo.Game).AddRemoteBot(
		self,
		clientName,
		teamNum,
		className,
		DesiredSkin,
		DesiredSkill,
		ShouldLeadTarget
	);

	//Here the spawning of the pawn is handled - after weve got controler created
	if(theBot != None) {
		FocusActor = Spawn(class'FocusActor',self);
		theBot.myTarget = Spawn(class'FocusActor',theBot);

		theBot.bAutoSpawn = true; //TODO HACK?
		if (GetArgVal("ManualSpawn")!="")
			theBot.bAutoSpawn = !bool(GetArgVal("ManualSpawn"));

		if (GetArgVal("AutoPickupOff") != "")
			theBot.bDisableAutoPickup = bool(GetArgVal("AutoPickupOff"));

		theBot.jmx = GetArgVal("Jmx");

		if (GetArgVal("AutoTrace")!="")
			theBot.bAutoTrace = bool(GetArgVal("AutoTrace"));

		theBot.ResetSkill(); //initialize difficulty and enable movement?
		SendNotifyConf(theBot);
		
        outstring = "INITED {BotId " $ GetUniqueId(theBot) $
       		"} {HealthStart " $ theBot.PawnClass.Default.Health $
       		"} {HealthFull " $ theBot.PawnClass.Default.HealthMax $
       		"} {HealthMax " $ theBot.PawnClass.Default.SuperHealthMax $
       		//"} {AdrenalineStart " $ int(thebot.Adrenaline) $
       		//"} {AdrenalineMax " $ int(thebot.AdrenalineMax) $
       		//"} {ShieldStrengthStart " $ int(class'xPawn'.Default.ShieldStrength) $
       		//"} {ShieldStrengthMax " $ int(class'xPawn'.Default.ShieldStrengthMax) $
       		//"} {MaxMultiJump " $ class'xPawn'.Default.MaxMultiJump $
       		"} {DamageScaling " $ theBot.PawnClass.Default.DamageScaling $
       		"} {GroundSpeed " $ theBot.PawnClass.Default.GroundSpeed $
       		"} {WaterSpeed " $ theBot.PawnClass.Default.WaterSpeed $
       		"} {AirSpeed " $ theBot.PawnClass.Default.AirSpeed $
       		"} {LadderSpeed " $ theBot.PawnClass.Default.LadderSpeed $
       		"} {AccelRate " $ theBot.PawnClass.Default.AccelRate $
       		"} {JumpZ " $ theBot.PawnClass.Default.JumpZ $
       		//"} {MultiJumpBoost " $ class'xPawn'.Default.MultiJumpBoost $
       		"} {MaxFallSpeed " $ theBot.PawnClass.Default.MaxFallSpeed $
       		//"} {DodgeSpeedFactor " $ theBot.PawnClass.Default.DodgeSpeedFactor $
       		//"} {DodgeSpeedZ " $ theBot.PawnClass.Default.DodgeSpeedZ $
       		"} {AirControl " $ theBot.PawnClass.Default.AirControl $
       		"}";

       	SendLine(outstring);

		if ( GetArgVal("Location")!="" )
		{
        	ParseVector(StartLocation,"Location");

        	if ( GetArgVal("Rotation")!="" )
			{
				ParseRot(StartRotation,"Rotation");
				BotDeathMatch(WorldInfo.Game).SpawnPawn(theBot,StartLocation,StartRotation);
			}
			else
			{
				BotDeathMatch(WorldInfo.Game).SpawnPawn(theBot,StartLocation, );
			}
		}
		else
		{
			if ( GetArgVal("Rotation")!="" )
			{
				ParseRot(StartRotation,"Rotation");
				BotDeathMatch(WorldInfo.Game).SpawnPawn(theBot, ,StartRotation);
			}
		}

		if (theBot.Pawn != none)
			theBot.GotoState('Alive', 'Begin');
		else
			theBot.GotoState('Dead', 'Begin');

		gotoState('monitoring','Running');
	}
	else
	{
		if (bDebug)
			`log("In InitReceived() - Error adding bot");
	}

}

//triggered when weve gotten ready message from client, or after succesfull password check
//we will send game nfo, all navpoints and all items in a map
function ExportStatus()
{
    SendLine("SHS"); //HandShake start
	if (bExportGameInfo)
		SendGameInfo();
	if (bExportMutators)
		ExportMutators(); //SMUT, MUT, EMUT messages
	if (bExportITC)
		ExportItemClasses(); //SITC, ITC, EITC messages
	if (bExportNavPoints)
		ExportNavPoints();
	if (bExportMovers)
		ExportMovers();
	/*if (bExportInventory)
		ExportInventory();*/ //temporary disabled
	if (bExportPlayers) {
		SendLine("SPLR");
		ExportPlayers(true); //true for limited info
		SendLine("EPLR");
	}
    SendLine("EHS"); //HandShake end
}

//Main function for processing commands
function ProcessAction(string cmdType)
{
	if(bDebug)
		`log("comandType:"@cmdType);

	if (theBot != none) {
		theBot.myRepInfo.SetMyLastGBCommand(lastGBCommand);
	}

	switch(cmdType)
	{
		case "CONF":
			ReceivedConf();
		break;
		case "CMOVE":
			ReceivedCMove();
		break;
		case "INIT":
			InitBot();
		break;
		case "GETPATH":
			ReceivedGetPath();
		break;
		case "JUMP":
			ReceivedJump();
		break;
		case "MESSAGE":
			ReceivedMessage();
		break;
		case "MOVE":
			ReceivedMove();
		break;
		case "PING":
			SendLine("Pong");
		break;
		case "QUIT":
			Closed();
		break;
		case "READY":
			ReceivedReady();
		break;		
		case "RESPAWN":
			ReceivedRespawn();
		break;
		case "ROTATE":
			ReceivedRotate();
		break;
		case "SETCROUCH":
			ReceivedSetCrouch();
		break;
		case "SETROUTE":
			ReceivedSetRoute();
		break;
		case "SETWALK":
			ReceivedSetWalk();
		break;
		case "SHOOT":
			ReceivedShoot();
		break;
		case "STOP":
			ReceivedStop();
		break;
		case "STOPSHOOT":
			ReceivedStopShoot();
		break;
		case "TURNTO":
			ReceivedTurnTo();
		break;
	}//end switch
}

function ReceivedConf()
{
	local float floatNumber;
	local string target;

	if (theBot == none)
		return;

	if (GetArgVal("AutoTrace") != "")
		theBot.bAutoTrace = bool(GetArgVal("AutoTrace"));
	if (GetArgVal("DrawTraceLines") != "")
		theBot.bDrawTraceLines = bool(GetArgVal("DrawTraceLines"));
	if (GetArgVal("ManualSpawn") != "")
		theBot.bAutoSpawn = !bool(GetArgVal("ManualSpawn"));
	if (GetArgVal("ShowFocalPoint")!="")
		theBot.bShowFocalPoint = bool(GetArgVal("ShowFocalPoint"));
	if (GetArgVal("ShowDebug")!="")
		theBot.bDebug = bool(GetArgVal("ShowDebug"));
	if (GetArgVal("Name") != "")
	{
		target = GetArgVal("Name");
		theBot.PlayerReplicationInfo.PlayerName = target;
		WorldInfo.Game.ChangeName( theBot, target, true );
	}
	if (GetArgVal("SpeedMultiplier") != "")
	{
		floatNumber = float(GetArgVal("Speed"));
		if ( (floatNumber >= 0.1) && (floatNumber <= theBot.MaxSpeed) )
		{
			theBot.SpeedMultiplier = floatNumber;
		    if (theBot.Pawn != none) {
				theBot.Pawn.GroundSpeed = floatNumber * theBot.Pawn.Default.GroundSpeed;
				theBot.Pawn.AirSpeed = floatNumber * theBot.Pawn.Default.AirSpeed;
				theBot.Pawn.WaterSpeed = floatNumber * theBot.Pawn.Default.WaterSpeed;
				theBot.Pawn.LadderSpeed = floatNumber * theBot.Pawn.Default.LadderSpeed;
		    }
		}
	}
	if ((GetArgVal("Invulnerable") != "") && (bAllowCheats == true))
		theBot.bGodMode = bool(GetArgVal("Invulnerable"));
	if (GetArgVal("VisionTime") != "")
	{
		floatNumber = float(GetArgVal("VisionTime"));
		if ((floatNumber >= 0.1) && (floatNumber <= 2))
			visionTime = floatNumber;
	}
	if (GetArgVal("SynchronousOff") != "")
	{
		theBot.myConnection.bSynchronousMessagesOff = bool(GetArgVal("SynchronousOff"));
	}
	if (GetArgVal("AutoPickupOff") != "")
	{
		theBot.bDisableAutoPickup = bool(GetArgVal("AutoPickupOff"));
		if (theBot.Pawn != none)
			theBot.Pawn.bCanPickupInventory = !theBot.bDisableAutoPickup;
	}
	SendNotifyConf(theBot);
}


function ReceivedCMove()
{
	local rotator yawRotation;

	if (theBot == None || theBot.Pawn == None)
		return;

	//We need to reset focus, otherwise the focus would reset focal point to its own location
	theBot.Focus = None;

	yawRotation.Yaw = theBot.Pawn.Rotation.Yaw;
    theBot.myFocalPoint = theBot.Pawn.Location + 500 * vector(yawRotation);
	theBot.GotoState('Alive','MoveContinuous');
}

function ReceivedGetPath()
{
	local vector v;
	local string id;
	local int i;

	if ( theBot == None )
		return;

	
	//clear the old path
	for ( i=0; i<theBot.RouteCache.Length; i++ )
	{
		if ( theBot.RouteCache[i] == None )
			break;
		else
		{
			theBot.RouteCache[i] = None;
		}
	}

	ParseVector(v,"Location");
	theBot.FindPathTo(v);
	id = GetArgVal("Id");

	SendLine("SPTH {MessageId " $ id $ "}");
	for ( i=0; i<theBot.RouteCache.Length; i++ )
	{
		if ( theBot.RouteCache[i] == None )
			break;
		else
		{
			SendLine("IPTH {RouteId " $ theBot.RouteCache[i] $"}");
		}
	}
	SendLine("EPTH");
}

function ReceivedJump()
{
	local string tmp;
	local bool bDouble;

	if (theBot == None || theBot.Pawn == None)
		return;

	bDouble = false;
	tmp = GetArgVal("DoubleJump");
	if (tmp != "")
		bDouble = bool(tmp);

	theBot.RemoteJump(bDouble);

}

function ReceivedMessage()
{
	local string target, text;
	local bool boolResult;
	local float FadeOut;

	if (theBot == None )
		return;

	//Note - currently only allow messages under 256 chars
	target = GetArgVal("Id");
	text = GetArgVal("Text");
	boolResult = bool(GetArgVal("Global"));
	FadeOut = float(GetArgVal("FadeOut"));
	if(text != "")
	{
		theBot.RemoteBroadcast(target,text,boolResult,FadeOut);
	}

}

function ReceivedMove()
{
	local vector v,v2,focusLoc;
	local string focusId;
	local Actor tmpFocus;
	local Controller C;
	local NavigationPoint N;

	if (theBot == None || theBot.Pawn == None)
		return;

	//if first location not specified, we wont move
	if (GetArgVal("FirstLocation")=="")
		return;

	ParseVector(v,"FirstLocation");
	focusId = GetArgVal("FocusTarget");
	//set the destinations we want to traverse
	theBot.myDestination = v;
	theBot.SetDestinationPosition(v);

	if (GetArgVal("SecondLocation")!="") {
		ParseVector(v2,"SecondLocation");
		theBot.pendingDestination = v2;
	} else {
		theBot.pendingDestination = v;
	}

	if(focusId == "") {
		if (GetArgVal("FocusLocation")!="") {
			ParseVector(focusLoc,"FocusLocation");
			//Cant focus to location, but can to actor, we set position of our helper actor to desired
	        FocusActor.SetLocation(focusLoc);
	        //set the bot focus to our helper actor
			theBot.Focus = FocusActor;
			//theBot.myFocus = FocusActor;
			//Set FocalPoint accordingly (it would change to desired values anyway)
			theBot.myFocalPoint = FocusActor.Location;
			theBot.SetFocalPoint(theBot.myFocalPoint);
		} else {
			//we reset old focus, if none focus set, the bot will turn towards destination
			theBot.Focus = none;
			//todo: reset also target?

			//set myFocalPoint to prevent unwanted turning back at the end of movement
			//myFocalPoint will be set at the end of movement
			if (GetArgVal("SecondLocation")!="")
				theBot.myFocalPoint = v2 + 500 * vector(rotator(v2 - v));
			else
				theBot.myFocalPoint = v + 500 * vector(rotator(v - theBot.Pawn.Location));
		}
	} else { //We have Id of the object we want to face
		//First we determine if it is a bot id
		tmpFocus = None;
	foreach WorldInfo.AllControllers(class'Controller', C) {
			if( GetUniqueId(C) == focusId) {
				if( C.Pawn != none && theBot.Pawn.LineOfSightTo(C.Pawn)) {
					tmpFocus = C.Pawn;
					break;
				}
			}
		}
		//we found it is a bot id, lets set it as our focus
		if (tmpFocus != none) {
			//point the bot at the location of the target
			theBot.SetFocalPoint(tmpFocus.Location);
			theBot.myFocalPoint = tmpFocus.Location;
			theBot.Focus = tmpFocus;
		} else { // it was not a bot id - try navigation points
			foreach WorldInfo.AllNavigationPoints(class'NavigationPoint', N){
				if( GetUniqueId(N) == focusId) {
					if(theBot.Pawn.LineOfSightTo(N)) {
						tmpFocus = N;
						break;
					}
				}		
			}	
			//lets try if there is an object with this id
			//can be navpoint or some of the items have unique id			
			//if succes, then set this object as our focus object
			if (tempActor != none) {
				theBot.Focus = tempActor;
				theBot.myFocalPoint = tempActor.Location;
				theBot.SetFocalPoint(tempActor.Location);
			}
		}
	}
	theBot.GotoState('Alive','Move');
}

function ReceivedRespawn() {
	local Vector v;
	local Rotator r;
	local bool hasVector, hasRotator;

	if (theBot != none ) {
		if (GetArgVal("StartLocation") != "") {
			ParseVector(v,"FirstLocation");
			hasVector = true;
		}
		if (GetArgVal("StartRotation") != "") {
			ParseRot(r, "StartRotation");
			hasRotator = true;
		}
		if (hasVector && hasRotator) {
			theBot.RespawnPlayer(v,r);
		} else if (hasVector) {
			theBot.RespawnPlayer(v);
		} else {
			theBot.RespawnPlayer();
		}
	}
}

function ReceivedReady()
{
	ExportStatus();
}

function ReceivedRotate()
{
	local string target;
	local rotator r;
	local int i;

	if (theBot.Pawn == None)
		return;

	target = GetArgVal("Axis");
	r = theBot.Pawn.GetViewRotation();
	i = int(GetArgVal("Amount"));
	if(target == "Vertical") {		
		r.Pitch += i;		
	} else {		
		r.Yaw += i;
	}

	theBot.myFocalPoint = theBot.Pawn.Location + ( vector(r) * 500);
	theBot.SetFocalPoint(theBot.myFocalPoint);
	theBot.Focus = None; //theBot.Pawn.Location + ( vector(r) * 1000);

    if (theBot.movingContinuous)
		theBot.GotoState('Alive','MoveContinuous');

	//We comment this so turning commands do not interupt moving commands.
	//theBot.StopWaiting();
	//theBot.GotoState('Startup', 'Turning');
}

function ReceivedSetCrouch()
{
	local string target;

	if (theBot == None || theBot.Pawn == None)
		return;

	target = GetArgVal("Crouch");
	
	theBot.Pawn.ShouldCrouch( bool(target) );

}

function ReceivedSetRoute()
{
	local vector v;
	local int i;

	if (theBot == None)
		return;

	if (bool(GetArgVal("Erase"))) {
		for (i=0;i<32;i++) {
			theBot.myRepInfo.SetCustomRoute(vect(0,0,0),i);
		}
	}

	for (i=0;i<32;i++) {
		ParseVector(v ,"Point"$i );
		theBot.myRepInfo.SetCustomRoute(v,i);
	}
}

function ReceivedSetWalk()
{

	local string target;

	if (theBot == None || theBot.Pawn == None)
		return;

	target = GetArgVal("Walk");
	
	theBot.Pawn.bIsWalking = bool(target);

}

function ReceivedShoot()
{
	local rotator r;
	local string Target;
	local Controller C;
	local vector v;
	local bool targetLocked;

	if (theBot == None || theBot.Pawn == None)
		return;

	targetLocked = false;
	theBot.bTargetLocationLocked = false;
	Target = GetArgVal("Target");
	if( Target != "") {
		foreach WorldInfo.AllControllers(class'Controller', C) {
		//We wont start shooting at non visible targets
			if( (GetUniqueId(C) == target) &&
				(C.Pawn != None) &&
				(theBot.LineOfSightTo(C.Pawn))
			)
			{
				//We will set desired bot as our enemy
				theBot.Focus = C.Pawn;
				theBot.Enemy = C.Pawn;
				theBot.RemoteEnemy = C;

				theBot.myFocalPoint = C.Pawn.Location;
				theBot.SetFocalPoint(C.Pawn.Location);							
				targetLocked = true;
				break;
			}
		}
	}

	if (!targetLocked && GetArgVal("Location") != "")
	{
		ParseVector(v,"Location");

		theBot.myTarget.SetLocation(v);
		theBot.bTargetLocationLocked = true;

		//We are shooting at a location. We will set the FocalPoint
		theBot.SetFocalPoint(theBot.myTarget.Location);
		theBot.myFocalPoint = theBot.myTarget.Location;

		theBot.Focus = theBot.myTarget;
		theBot.Enemy = None;
	}

	theBot.RemoteFireWeapon(bool(GetArgVal("Alt")));
}

function ReceivedStop()
{
	if (theBot == None || theBot.Pawn == None)
		return;

	theBot.GotoState('Alive', 'DoStop');
}

function ReceivedStopRec()
{
	ConsoleCommand("stopdemo", True);
	SendLine("RECEND");
}

function ReceivedStopShoot()
{
	if (theBot == None)
		return;

	theBot.shouldFire = 0;
	theBot.StopFiring();
}


function ReceivedTurnTo()
{
	local Controller C;
	local NavigationPoint N;
	local vector v;
	local rotator r;
	local string target;

	if (theBot == none || theBot.Pawn == None)
		return;

	target = GetArgVal("Target");
	if(target == "") {
		ParseRot(r,"Rotation");
		if(r.Yaw == 0 && r.Pitch == 0 && r.Roll == 0) {
			//no target or rotation defined
			ParseVector(v,"Location");
			theBot.SetFocalPoint(v);
			theBot.myFocalPoint = v;
			//We erase possible focus actors
			theBot.Focus = None;

			if (theBot.movingContinuous)
				theBot.GotoState('Alive','MoveContinuous');
		} else {
			//no target, yes rotation
			theBot.myFocalPoint = theBot.Pawn.Location + ( vector(r) * 500);
			theBot.SetFocalPoint(theBot.myFocalPoint);

			//We erase possible focus actors
			theBot.Focus = None;

			if (theBot.movingContinuous)
				theBot.GotoState('Alive','MoveContinuous');
		}
	} else {
		//target defined
		//First we try to find if we should focus to a player or bot
		foreach WorldInfo.AllControllers(class'Controller', C) {
			if( target == GetUniqueId(C) ) {
				break;
			}
		}//end for

		if (C != None) {
			//Pawn must exists and must be visible
			if ((C.Pawn != None) && theBot.Pawn.LineOfSightTo(C.Pawn)) {
				//We set the Controller as our target
				theBot.SetFocalPoint(C.Pawn.Location);
				theBot.myFocalPoint = C.Pawn.Location;
				theBot.Focus = C.Pawn;

				if (theBot.movingContinuous)
					theBot.GotoState('Alive','MoveContinuous');

			} else {
				return;
			}
		} else {
			tempActor = none;
			//try navigation points
			foreach WorldInfo.AllNavigationPoints(class'NavigationPoint', N){
				if( GetUniqueId(N) == target) {
					if(theBot.Pawn.LineOfSightTo(N)) {
						tempActor = N;
						break;
					}
				}		
			}	
			
			//Actor must be visible
			if ((tempActor != None) && theBot.Pawn.LineOfSightTo(tempActor)) {
				theBot.Focus = tempActor;
				theBot.myFocalPoint = tempActor.Location;
				theBot.SetFocalPoint(theBot.myFocalPoint);

				if (theBot.movingContinuous)
					theBot.GotoState('Alive','MoveContinuous');
			} else {
				return;
			}
		}
	}
}

//----------------- STATES

//Waiting state when connected
auto state waiting
{
Begin:
Waiting:
	sleep(5.0);
	goto 'Waiting';

}

state checkingPassword
{
Begin:
Waiting:
	sleep(5.0);
	goto 'Waiting';
}

state monitoring
{

Begin:
Running:
	if (bSynchronousMessagesOff)
		goto 'SynchronousOff';
	if((theBot != none) && !theBot.IsInState('Dead') && !theBot.IsInState('GameEnded') )
	{
		//This is where synchronous batch messages are sent
		SendLine("BEG {Time " $ WorldInfo.TimeSeconds $"}");

		//`log("Measuring Time of synchronous batch: ");
		//StopWatch(false);

       // ExportGameStatus();
		theBot.checkSelf();
		theBot.checkVision();
		//StopWatch(true);
		SendLine("END {Time " $ WorldInfo.TimeSeconds $"}");
	}
	//theBot.bDebug = true;
	if (theBot != none && theBot.bDebug && !theBot.IsInState('Alive'))
		`log(theBot.getStateName());
	sleep(visionTime);
	goto 'Running';
SynchronousOff:
	if (!bSynchronousMessagesOff)
		goto 'Running';
	sleep(1);
	goto 'SynchronousOff';
}


