/*
Gamebots UT Copyright (c) 2002, Andrew N. Marshal, Gal Kaminka
Gamebots Pogamut derivation Copyright (c) 2010, Michal Bida

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This software must also be in compliance with the Epic Games Inc. license for mods which states the following: "Your mods must be distributed solely for free, period. Neither you, nor any other person or party, may sell them to anyone, commercially exploit them in any way, or charge anyone for receiving or using them without prior written consent of Epic Games Inc. You may exchange them at no charge among other end-users and distribute them to others over the Internet, on magazine cover disks, or otherwise for free." Please see http://www.epicgames.com/ut2k4_eula.html for more information.

*/


//=============================================================================
// RemoteBot.
// Basedon Bot class
//=============================================================================
class RemoteBot extends UTBot
	config(GameBotsUDK);

var config bool bDebug;

//maybe link to bot's periphreal vision? UT bots need much less of an arc though
//like Pawn's PeriphrealVision, this value should be the cosine of the limits
//of visual field. (i.e. 0.707 = 45 degrees to each side, or a 90* arc.)
var config float remoteVisionLimit;

//The socket to the agent
var BotConnection myConnection;

//The three remote vars compliment the my vars right below. The only one
//that ever needs to be duplicated is RemoteEnemy and myTarget
//just need RemoteDestination || myDestination and RFocus || myFocus

//Who the remote bot is trying to shoot at. Used by the aiming code.
var Actor RemoteEnemy;

//The spot the bot is shooting at
var FocusActor myTarget;

//If false, we will update myTarget location according our current focal point
var bool bTargetLocationLocked;

//The spot the bot is moving to
var vector myDestination;
//The spot the bot is looking at
var vector myFocalPoint;

//Used for smooth movement, see StartUp state
var vector pendingDestination;

var class<UTPawn> PawnClass;

//This is an indicator that we are in StartUp:MoveContinuous state. We need this, so turn
//command work properly when the bot is moving continuous!
var bool movingContinuous;

//If true bot will respawn automatically when killed
var config bool bAutoSpawn;

//When true auto raytracing producing sync. ATR messsages will be on
var config bool bAutoTrace;

//If true we will draw trace lines (results of AutoTrace() functionsin BotConnection)
//in the game
var config bool bDrawTraceLines;

//If we should spawn an actor at the point the bot has set as focal point
var config bool bShowFocalPoint;

//If we should provide aim correct based on bot Skill also when shooting at Location
var config bool bPerfectLocationAim;

//If we should include FadeOut attribute into message text or not. for debug.
var config bool bIncludeFadeOutInMsg;

//maximum number we can multiply BaseSpeed of the bot
var config float MaxSpeed;

//Used for changing the bot appearance. see GBxPawn.Setup() function
var config string DesiredSkin;

//TODO: Probably not used anymore - delete
var bool bSecondJumpPossible;

//For disabling auto pickup, custom varialbe, will affect Pawn.bCanPickupInventory
var config bool bDisableAutoPickup;

//By this default pawn speed will be multiplied ranges from 0.1 to 2
var config float SpeedMultiplier;

//helper variable to store the direction we want to go with CMOVE
var vector cmoveDirection;

//this is here, so we can properly debug the bot in java. it is the adress we need
//to connect to in order to debug the bot (in Pogamut the bots are now run each one in
//different JVM)
var string jmx;

//holding last bump time to avoid sending the message repeatably
var float lastBumpTime;

//holding last wall hit to avoid sending the message repeatably
var float lastWallHit;

//1 - normal fire, 2 - alt fire, 0 - should not fire
var byte shouldFire;

var GBReplicationInfo myRepInfo;

//override - using translocator in this function
function Actor FaceActor(float StrafingModifier)
{
	return none;
}

//Called at the start of the match
function StartMatch()
{
	// SENDMESSAGE
	// !!! possible new message
}


//This function will properly destroy the pawn of this conroller (otherwise conflicts with game mechanics)
function DestroyPawn()
{
	if (Pawn == None)
		return;

	Pawn.Died(none, class'DamageType', Pawn.Location);
}

//This function is called by the RESPAWN command - it makes sure everything will work ok
function RespawnPlayer(optional vector startLocation, optional rotator startRotation)
{
	if (IsInState('Dead') && !bAutoSpawn) //
	{
		RemoteRestartPlayer(startLocation,startRotation);
	}
	else if (bAutoSpawn)
	{
		bAutoSpawn = false; //otherwise it would get respawned in Dead state without specifyed locatin, rotation
		DestroyPawn();
		GotoState('Dead');
		RemoteRestartPlayer(startLocation,startRotation);
		bAutoSpawn = true;

	}
	else  if (!IsInState('Dead') && !bAutoSpawn)
	{
		DestroyPawn();
		GotoState('Dead');
		RemoteRestartPlayer(startLocation,startRotation);
	}
}

//This function handles actual bot respawning - when it dies for instance
function RemoteRestartPlayer(optional vector startLocation, optional rotator startRotation) {
	//Pawn = Spawn(class'UTGame.UTPawn',self,,,); 
	`log("Spawn hack");
	
	if( WorldInfo.Game.isA('BotDeathMatch')) {
		BotDeathMatch(WorldInfo.Game).RemoteRestartPlayer( self, startLocation, startRotation );
	}

	if (Pawn != None) {
		gotostate('Alive','DoStop');
	} else {
		`log("Pawn is None after ResartPlayer");
	}
}

function HandlePickup(Inventory Inv) {
	//myConnection.SendLine("IPK {Id " $ myConnection.GetUniqueId(Inv) $ "}");

}

//Called when the bot land on the ground after falling
function bool NotifyLanded(vector HitNormal, Actor FloorActor)
{
	myConnection.SendLine("LAND {HitNormal " $ HitNormal $ "}");

	//restart, so we continue moving and not going back to our previous
	//destination, as we may be over it because of the fall
	if (movingContinuous)
		gotoState('Alive','MoveContRepeat');
	return true;
}

// Called when get new weapon or ammunition for weapon we do not have yet.
// Called just once per weapon type or per new ammunition type (notify new object in our inventory, NOT pickup)
// For exporting weapons we have new attributes.
function NotifyAddInventory( Inventory inputInventory )
{
	local string outstring;

	//For weapons
	local Weapon inputWeapon;
	local bool bUseAlternateAmmo;

	Super.NotifyAddInventory(inputInventory);

    outstring = "AIN {Id " $ myConnection.GetUniqueId(inputInventory) $
		"} {Type " $ inputInventory.class $
		
		//"} {PickupType " $ inputInventory.DroppedPickupClass $
		"}";

	//myConnection.SendLine(outstring);
}

function NotifyTakeHit(Controller InstigatedBy, vector HitLocation, int Damage, class<DamageType> damageType, vector Momentum)
{
	local string outstring;

	LastUnderFire = WorldInfo.TimeSeconds;

	outstring = "DAM {Damage " $ Damage $ 
		"} {DamageType " $ damageType $ 
		"}";
	if (InstigatedBy != none && inFOV(InstigatedBy.Location)) {
		outstring $= " {Instigator " $ myConnection.GetUniqueId(InstigatedBy) $ "}";
	}

	myConnection.SendLine(outstring);
	//if ( (instigatedBy != None) && (instigatedBy != self) )
	//	damageAttitudeTo(instigatedBy, Damage);
}

function NotifyKilled(Controller Killer, Controller Killed, pawn KilledPawn, class<DamageType> damageType)
{	
	local string outstring;
	
	if (Killed == self) {
		outstring = "DIE {DamageType " $ damageType $ "}";			
	} else {
		outstring = "KIL {Id " $ myConnection.getUniqueId(Killed) $ 
			"} {DamageType " $ damageType $ "}";
	}
	if (Killer != none) 
		outstring $= " {Killer " $ myConnection.getUniqueId(Killer) $ "}";

	myConnection.SendLine(outstring);
}

event HearNoise(float Loudness, Actor NoiseMaker, optional Name NoiseType )
{

	myConnection.SendLine("HRN {Source " $ myConnection.GetUniqueId(NoiseMaker) $
		"} {Type " $ NoiseMaker.Class $ 
		//"} {Rotation " $ $ //How should bot rotate if it would like to be in the direction of the "noisy" actor.
		"}");
}

//overriding! so no code gets executed
event SeePlayer(Pawn SeenPlayer)
{

}

event bool NotifyHitWall(vector HitNormal, actor Wall)
{

	if ( WorldInfo.TimeSeconds - 0.3 >= lastWallHit)
	{

		myConnection.SendLine("WAL {Id " $ myConnection.GetUniqueId(Wall) $
			"} {Normal " $ HitNormal $ 
			"} {Location " $ Wall.Location $ "}");

		lastWallHit = WorldInfo.TimeSeconds;
	}

	return true;
}

event NotifyPostLanded()
{
	bNotifyPostLanded = false;
}

//overriding! so no code gets executed 
event NotifyMissedJump()
{

}
//overriding! so no code gets executed 
event NotifyJumpApex()
{

}
//overriding! so no code gets executed 
event MissedDodge()
{

}

//handles sending messages to all and to team
function RemoteBroadcast( string Id, coerce string Msg, bool bGlobal, float FadeOut )
{

	local Controller C;
	local string MsgForVoice;

	MsgForVoice = Msg;
	if (bIncludeFadeOutInMsg)
		Msg = Msg $ " FadeOut=" $ FadeOut $ " s.";


    //Set the text bubble, which will be shown on the HUDs of Human players
    //when they see the bot
    if (Pawn != none)
   	{
   		//if (bSpeakingBots)
	   		//Pawn.TextToSpeech( MsgForVoice, 10 );
		//GBxPawn(Pawn).SetTextBubble(Id, Msg, bGlobal, FadeOut);
	}

	if (Id != "")
	{

		//If we haven't found Id or Id doesn't belong to RemoteBot we will end
		return;
	}

	if ( bGlobal || !WorldInfo.Game.bTeamGame )
	{
	    //Send the message to the game channel
	    WorldInfo.Game.Broadcast(self, Msg, 'Say');
	    //Send the message to RemoteBots
	    SendGlobalMessage( Msg );
	    return;
	}

	if (WorldInfo.Game.bTeamGame)
	{
		//Send the message to team channel
		WorldInfo.Game.BroadcastTeam(self, Msg, 'TeamSay');
		//Send the message to RemoteBots
		SendTeamMessage( Msg );
		return;
	}
}

//Event for receiving string messages, called manually
event RemoteNotifyClientMessage( Controller C, coerce string S )
{
	//Could cause some parsing problems, so replacing
	S = Repl(S, "{", "_", false );
	S = Repl(S, "}", "_", false );

	myConnection.SendLine("VMS {Id "$ myConnection.GetUniqueId(C) $
		"} {Name " $ C.PlayerReplicationInfo.PlayerName $
		"} {Text " $ S $"}");
}

//Event for receiving string messages, called manually
event RemoteNotifyTeamMessage( Controller C, coerce string S )
{
	//Could cause some parsing problems, so replacing
	S = Repl(S, "{", "_", false );
	S = Repl(S, "}", "_", false );

	myConnection.SendLine("VMT {Id "$ myConnection.GetUniqueId(C) $
		"} {Name " $ C.PlayerReplicationInfo.PlayerName $
		"} {Text " $ S $"}");
}

//Send the team message to RemoteBots
function SendTeamMessage( string S )
{
	local Controller C;

}

//Send the global message to RemoteBots
function SendGlobalMessage( string S )
{
	local Controller C;

}

/* SetOrders()
Called when player gives orders to bot
*/
function SetBotOrders(name NewOrders, Controller OrderGiver, bool bShouldAck) {}

//The bot (or some part of the bot - legs) entered new volume
event NotifyPhysicsVolumeChange( PhysicsVolume NewVolume )
{

	//this code is taken from super class (Bot). Had to disable automatic jumping out of water
	//super class is not called anymore
	if ( newVolume.bWaterVolume )
	{
		bPlannedJump = false;
		if (!Pawn.bCanSwim)
			MoveTimer = -1.0;
		else if (Pawn.Physics != PHYS_Swimming)
			Pawn.setPhysics(PHYS_Swimming);
	}
	else if (Pawn.Physics == PHYS_Swimming)
	{
		if ( Pawn.bCanFly )
			 Pawn.SetPhysics(PHYS_Flying);
		else
		{
			Pawn.SetPhysics(PHYS_Falling);
			/*if ( Pawn.bCanWalk && (Abs(Pawn.Acceleration.X) + Abs(Pawn.Acceleration.Y) > 0)
				&& (Destination.Z >= Pawn.Location.Z)
				&& Pawn.CheckWaterJump(jumpDir) )
			{
				Pawn.JumpOutOfWater(jumpDir);
				bNotifyApex = true;
				bPendingDoubleJump = true;
			}*/
		}
	}

	myConnection.SendLine("VCH {Id " $ myConnection.GetUniqueId(NewVolume) $
		"} {ZoneVelocity " $ NewVolume.ZoneVelocity $  //vector
		//"} {ZoneGravity " $ NewVolume.Gravity $  //vector
		//"} {GroundFriction " $ NewVolume.GroundFriction $  //float
		//"} {FluidFriction " $ NewVolume.FluidFriction $  //float
		//"} {TerminalVelocity " $ NewVolume.TerminalVelocity $  //float
		//"} {WaterVolume " $ NewVolume.bWaterVolume $
		//"} {PainCausing " $ NewVolume.bPainCausing $
		//"} {Destructive " $ NewVolume.bDestructive $
		//"} {DamagePerSec " $ NewVolume.DamagePerSec $ //float
		//"} {DamageType " $ NewVolume.DamageType $
		//"} {NoInventory " $ NewVolume.bNoInventory $
		//"} {MoveProjectiles " $ NewVolume.bMoveProjectiles $
		//"} {NeutralZone " $ NewVolume.bNeutralZone $
		"}");


}

//overriding so no code gets executed
event MayDodgeToMoveTarget()
{

}


//called on collisions with other actors
event bool NotifyBump(Actor Other, Vector HitNormal)
{
	local vector VelDir, OtherDir;
	local float speed;


	speed = VSize(Velocity);
	if ( speed > 10 )
	{
		VelDir = Velocity/speed;
		VelDir.Z = 0;
		OtherDir = Other.Location - Location;
		OtherDir.Z = 0;
		OtherDir = Normal(OtherDir);
		if ( (VelDir Dot OtherDir) > 0.8 )
		{
			Velocity.X = VelDir.Y;
			Velocity.Y = -1 * VelDir.X;
			Velocity *= FMax(speed, 280);
		}
	}

	if ( WorldInfo.TimeSeconds - 0.3 >= lastBumpTime )
	{

		myConnection.SendLine("BMP {Id " $ myConnection.GetUniqueId(Other) $
			"} {Location " $ Other.Location $"}");

		lastBumpTime = WorldInfo.TimeSeconds;
	}

	// Need to disable bumping ???
	//Disable('Bump');
	//TODO: Experiment with this?
	return false;
}



/* Added func RemoteKilled - should make things cleaner, called from GameTypeGame server
	25.10.2006 Michal Bida
*/
function RemoteKilled(Controller Killer, Controller Killed, Pawn KilledPawn, class<DamageType> damageType)
{
	local string outstring;

	outstring = "KIL {Id " $ myConnection.GetUniqueId(Killed) $
		"} {KilledPawn " $ KilledPawn $
        "} {Killer " $ myConnection.GetUniqueId(Killer) $
		"}";
	if (CanSee(KilledPawn))
	{
		outstring = outstring $ " {DamageType " $ damageType $
        //"} {DeathString " $ damageType.default.DeathString $
        //"} {WeaponName " $ damageType.default.DamageWeaponName $
        //"} {Flaming " $ damageType.default.bFlaming $
        //"} {CausedByWorld " $ damageType.default.bCausedByWorld $
        //"} {DirectDamage " $ damageType.default.bDirectDamage $
        //"} {BulletHit " $ damageType.default.bBulletHit $
        //"} {VehicleHit " $ damageType.default.bVehicleHit $
		"}";
	}

	myConnection.sendLine(outstring);
}

/* Added func RemoteDied - should make things cleaner, called from GameTypeGame server
	25.10.2006 Michal Bida
*/
function RemoteDied(Controller Killer, class<DamageType> damageType)
{
	myConnection.SendLine("DIE {Killer " $ myConnection.GetUniqueId(Killer) $
		"} {DamageType " $ damageType $
        //"} {DeathString " $ damageType.default.DeathString $
        //"} {WeaponName " $ damageType.default.DamageWeaponName $
        //"} {Flaming " $ damageType.default.bFlaming $
        //"} {CausedByWorld " $ damageType.default.bCausedByWorld $
        //"} {DirectDamage " $ damageType.default.bDirectDamage $
        //"} {BulletHit " $ damageType.default.bBulletHit $
        //"} {VehicleHit " $ damageType.default.bVehicleHit $
		"}");
}

//Jumps a bot
function RemoteJump(bool bDouble)
{
	if (Pawn == none)
		return;

	Pawn.DoJump(true);
}

//Intercept FireWeapon - is called from other code
function bool FireWeaponAt(Actor A) {
}

function RemoteFireWeapon(bool bUseAltMode) {

	if ((Pawn == none) || (Pawn.Weapon == none))
		return;

	if (bUseAltMode) {
		shouldFire = 2;
		Pawn.Weapon.StartFire(1);
	} else {
		shouldFire = 1;
		Pawn.Weapon.StartFire(0);
	}
}

//Function which determines if our weapon should fire again or stop firing
function bool WeaponFireAgain(bool bFinishedFire) { 
	if (Pawn == none || Pawn.Weapon == none || !Pawn.Weapon.HasAnyAmmo())
		return false;

	//`log("In WeaponFireAgain");
	if (shouldFire == 1) {
		Pawn.Weapon.StartFire(0);
	} else if (shouldFire == 2) {
		Pawn.Weapon.StartFire(1);
	}

	return false;
}


function bool TryToDuck(vector duckDir, bool bReversed) {

}

//Pointless callback
function EnemyAcquired();

//All kinds of things can call this mostly special trigger points
function Trigger( actor Other, pawn EventInstigator )
{
	myConnection.SendLine("TRG {Actor " $ myConnection.GetUniqueId(Other) $
		"} {EventInstigator " $ myConnection.GetUniqueId(EventInstigator) $
		"}");
}


//Don't let engine pick nodes that must be impact jumped
function bool CanImpactJump()
{
	return false;
}

//Don't handle impact jumps or low gravity manuevers for bots
/** performs an impact jump; assumes the impact hammer is already equipped and ready to fire */
//function ImpactJump();


function SetFall()
{
	if (Pawn.bCanFly)
	{
		Pawn.SetPhysics(PHYS_Flying);
		return;
	}
	if (bDebug)
		`log("In RemoteBot.uc: SetFall() enganged.");
	/*
	if ( Pawn.bNoJumpAdjust )
	{
		Pawn.bNoJumpAdjust = false;
		return;
	}
	else
	{
		bPlannedJump = true;
		Pawn.Velocity = EAdjustJump(Pawn.Velocity.Z,Pawn.GroundSpeed);
		Pawn.Acceleration = vect(0,0,0);
	} */
}


//**********************************************************************************
//Base RemoteBot AI, that controls the Pawn (makes him move)

auto state Alive {
	function BeginState(Name PreviousStateName) {
		`log("Alive,BeginState");
		ResetSkill();
		Pawn.SetMovementPhysics();
		Reset();
	}

	function EndState(Name NextStateName) {
		`log("Alive,EndState");
	}

Begin:
	movingContinuous = false;
	sleep(0.5);
	if (VSize(Pawn.Velocity) > 10) {//HACK - if the bot can't reach the destination, he would continue running - bad behavior
		StopMovement();
	}
	goto 'Begin';
DoStop:
	movingContinuous = false;
	StopMovement();
	goto 'Begin';
Move:
	//`log("Alive,Move:, MoveTo( " $ myDestination $ ")");
	movingContinuous = false;
	//SetDestinationPosition(myDestination);
	if (Focus == none) {
		MoveTo(myDestination,,,Pawn.bIsWalking);
		//MoveTo( pendingDestination, , ,);

		//There is an issue when the bot finish its movement, sometimes he goes a bit
		//over the target point, this caused turning back, because moveTo functions sets focalpoint
		//after it ends to its target point, to prevent this, we will set our own FocalPoint counted in advance
		SetFocalPoint(myFocalPoint);
	} else {
		MoveTo(myDestination, Focus,,Pawn.bIsWalking);
		//MoveTo( pendingDestination, Focus, , );
	}
	goto 'Begin';	
MoveToActor:
	movingContinuous = false;
	if (Focus == none) {
		MoveToward(MoveTarget, , , , Pawn.bIsWalking);
		//There is an issue when the bot finish its movement, sometimes he goes a bit
		//over the target point, this caused turning back, because moveTo functions sets focalpoint
		//after it ends to its target point, to prevent this, we will set our own FocalPoint counted in advance
		SetFocalPoint(myFocalPoint);
	} else {
		MoveToward(MoveTarget, Focus, , true, Pawn.bIsWalking);
	}
	goto 'Begin';
MoveContinuous:
	//to prevent that our focal point moves too much above or below us
	//remember we want to move
	myFocalPoint.z = Pawn.Location.z;
	cmoveDirection = vector(rotator(myFocalPoint - Pawn.Location));
MoveContRepeat:
	movingContinuous = true;
	MoveTo(Pawn.Location + 500 * cmoveDirection);

	myFocalPoint = Pawn.Location + 500 * cmoveDirection;
	SetFocalPoint(myFocalPoint);
	goto 'MoveContRepeat';
}

// This state was called somehow on our bot. That is highly undersirable.
// Overriding
state Roaming
{
	function BeginState(Name PreviousStateName)
	{
		`log("In Roaming STATE! Shouldnt be!");
		gotostate('Alive','Begin');
	}
Begin:
	gotostate('Alive','Begin');
}

state TakeHit
{
	//ignores seeplayer, hearnoise, bump, hitwall;

	function Timer();

Begin:
	//error("!!!TakeHit");
}

state GameEnded
{
ignores SeePlayer, EnemyNotVisible, HearNoise, TakeDamage, Bump, Trigger, HitWall, Falling, ReceiveWarning;

	function SpecialFire()
	{
	}
	function bool TryToDuck(vector duckDir, bool bReversed)
	{
		return false;
	}
	function SetFall()
	{
	}
	function LongFall()
	{
	}
	function Killed(pawn Killer, pawn Other, name damageType)
	{
	}
	function ClientDying(class<DamageType> DamageType, vector HitLocation)
	{
	}

	function BeginState(Name PreviousStateName)
	{
		`log("In GameEnded! BeginState");
		//Pawn.SimAnim.AnimRate = 0.0;
		bFire = 0;
		//bAltFire = 0;

		SetCollision(false,false,false);
		SetPhysics(PHYS_None);
		Velocity = vect(0,0,0);
		myConnection.SendLine("FIN");
	}
}

state Dead
{
ignores SeePlayer, HearNoise, KilledBy;

function BeginState(Name PreviousStateName)
{
	`log("State: Dead, fc BeginState()");

	//We can be sent to this state sometimes, when it is not desired ( by game mechanics)
	//Escaping here
	if (Pawn != none)
		gotostate('Alive','Begin');

	//This is taken from working AI in UT2004, probably needed to assure bot
	//will behave normally after restart - 02/03/07 Michal Bida
	movingContinuous = false;
	Enemy = None;
	Focus = None;
	RouteGoal = None;
	MoveTarget = None;
	shouldFire = 0;
	//Pawn = Spawn(class'UTGame.UTPawn',self,,,);

	Super.StopFiring(); //Not needed anymore to call super, but for sure.
/*	FormerVehicle = None;
	bFrustrated = false;
	BlockedPath = None;
	bInitLifeMessage = false;
	bPlannedJump = false;
	bInDodgeMove = false;
	bReachedGatherPoint = false;
	bFinalStretch = false;
	bWasNearObjective = false;
	bPreparingMove = false;
	bEnemyEngaged = false;
	bPursuingFlag = false;
*/


}
/*
function EndState()
{
	myConnection.sendLine("SPW2");
}*/
Begin:
	`log("In Dead:Begin:");
	//AutoSpawn policy
	if (Pawn != None)
		gotostate('Alive','Begin');
	//RemoteRestartPlayer(); //HACK
	if (bAutoSpawn)// && !WorldInfo.Game.bWaitingToStartMatch)
	{
		RemoteRestartPlayer();
	//	if (Pawn == none) //bug during restart?
	//		goto('Begin');
		//ServerRestartPlayer(); //Dunno if this is needed here - Could it cause troubles?
	}

	if (Pawn != None)
		gotostate('Alive','Begin');
/*	if (!WorldInfo.Game.bWaitingToStartMatch)
	{
		RemoteRestartPlayer();
	}
	if (Pawn != None)
		gotostate('StartUp','Begin');*/
	sleep(1.0);
	goto('Begin');

}



//-------------RemoteBot Specific Functions--------------------


// True if location loc is in bot's field of view. Does not take into account occlusion by geometry!
// Possible optimization: Precompute cos(obsController.FovAngle / 2) for InFOV - careful if it can change.
function bool InFOV(vector loc) {
	local vector view;   // vector pointing in the direction obsController is looking.
	local vector target; // vector from obsController's position to the target location.
	local Vector PawnLocation;

	if (Pawn == none) {
		return false;
	}

	view = vector(self.Pawn.GetViewRotation());
	PawnLocation = Pawn.Location;
	PawnLocation.Z += Pawn.EyeHeight;
	target = loc - PawnLocation;

	return ( Normal(view) dot Normal(target) ) < Pawn.PeripheralVision; // Angle between view and target is less than FOV
	// 57.2957795 = 180/pi = 1 radian in degrees  --  convert from radians to degrees
}


//Called by the gametype when someone else is injured by the bot
//TODO: From old gamebots, is not called anymore, TO REMOVE - mb
/*
function int HurtOther(int Damage, name DamageType, pawn injured)
{
	myConnection.SendLine("HIT" $ib$as$ "Id" $ib$ injured $ae$ib$as$
		"Damage" $ib$ Damage $ae$ib$as$
		"DamageType" $ib$ DamageType $ae);
} */

function checkSelf() {
	local string outstring, TeamIndex, altFiring;
	local rotator PawnRotation;
	local bool bIsShooting;

	local vector FloorLocation, FloorNormal;

	if(WorldInfo.Game.isA('BotTeamGame')) {
		TeamIndex = string(PlayerReplicationInfo.Team.TeamIndex);
	} else {
    	TeamIndex = "255";
	}
	if( Pawn != none ) {
		if( Pawn.Weapon != None)
		    bIsShooting = Pawn.Weapon.IsFiring();

		PawnRotation = Pawn.Rotation;
		//PawnRotation.Pitch = int(Pawn.ViewPitch) * 65556/255;

		FloorNormal = vect(0,0,0);
		Trace(FloorLocation,FloorNormal,Pawn.Location + vect(0,0,-1000),Pawn.Location, false);

		outstring = "SLF {Id " $ myConnection.GetUniqueId(self) $
			"} {Vehicle False" $
			"} {Rotation " $ PawnRotation $
			"} {Location " $ Pawn.Location $
			"} {Velocity " $ Pawn.Velocity $
			"} {Name " $ PlayerReplicationInfo.PlayerName $
			"} {Team " $ TeamIndex $
			"} {Health " $ Pawn.Health $
			"} {Weapon " $ Pawn.Weapon $
			"} {Shooting " $ bIsShooting $
			//"} {Armor " $ int(Pawn.ShieldStrength) $
			//"} {SmallArmor " $ int(xPawn(Pawn).SmallShieldStrength) $
			//"} {Adrenaline " $ int(Adrenaline) $
			"} {Crouched " $ Pawn.bIsCrouched $
			"} {Walking " $ Pawn.bIsWalking $
			"} {FloorLocation " $ FloorLocation $
			"} {FloorNormal " $ FloorNormal $
			//"} {Combo " $ xPawn(Pawn).CurrentCombo $
			//"} {UDamageTime " $ xPawn(Pawn).UDamageTime $
			"}";


		if( Pawn.Weapon != None && Pawn.Weapon.IsA('UDKWeapon')) {
			outstring = outstring $" {PrimaryAmmo "$ UDKWeapon(Pawn.Weapon).AmmoCount $ 
				"} {SecondaryAmmo "$ UDKWeapon(Pawn.Weapon).AmmoCount $
				"}";

			/*if (Pawn.Weapon.HasAnyAmmo())
				outstring = outstring $" {PrimaryAmmo 1} {SecondaryAmmo 0}";
			else
				outstring = outstring $" {PrimaryAmmo 0} {SecondaryAmmo 0}";
			*/
/*
			if( Pawn.Weapon.GetFireMode(1).IsFiring() )
			{
				outstring = outstring $" {AltFiring True}";
			}
			else
			{
				outstring = outstring $" {AltFiring False}";
			}*/
		}

        //`log("Debug: bFire "$bFire$" bAltFire "$bAltFire);

		myConnection.sendLine(outstring);
	} else {
		`log("Pawn is none in CheckSelf() ");
	}
}

function checkVision()
{
	local PickupFactory Pickup; //Respawned Inventory on the map is now Pickup;
	local Controller C;
	local NavigationPoint N;
	local InterpActor IA; //mover
	local int temp;
	local string outstring, TeamIndex, WeaponClass;
	local bool flagVisible;

	//for PRJ
	local Projectile Proj;
	local vector FireDir;
	local float projDistance;

	if( Pawn == none )
	{
		`log("In CheckVision() - Pawn is none ");
		return;
	}

	foreach WorldInfo.AllControllers(class'Controller', C)
	{
		if( C != self && C.Pawn != none && CanSee(C.Pawn))//to match peripheral vision
		{
			if(WorldInfo.Game.isA('BotTeamGame')) {
				TeamIndex = string(C.PlayerReplicationInfo.Team.TeamIndex);
			} else {
    			TeamIndex = "255";
			}
			if (C.Pawn.Weapon != none) {
				WeaponClass = string(C.Pawn.Weapon.Class);
			} else {
				WeaponClass = "None";
			}
			outstring = "PLR {Id " $ myConnection.GetUniqueId(C) $
				"} {Rotation " $ C.Pawn.Rotation $
				"} {Location " $ C.Pawn.Location $
				"} {Velocity " $ C.Pawn.Velocity $
				"} {Name " $ C.PlayerReplicationInfo.PlayerName $
				"} {Team " $ TeamIndex $
				"} {Reachable " $ actorReachable(C.Pawn) $
				"} {Weapon " $ WeaponClass $
				"}";

			if((C.Pawn.Weapon != none) && C.Pawn.Weapon.IsFiring() )
				outstring = outstring $" {Firing 1}";
			//else if((C.Pawn.Weapon != none) && C.Pawn.Weapon.GetFireMode(1).IsFiring() )
			//	outstring = outstring $" {Firing 2}";
			else
				outstring = outstring $" {Firing 0}";

			myConnection.sendLine(outstring);

        }//end if
	}//end for P=WorldInfo.ControllerList

	foreach AllActors(class'InterpActor', IA) {
		outstring = "MOV {Id " $ myConnection.GetUniqueId(IA) $ 
			"} {Location " $ IA.Location $ 
			"} {Velocity " $ IA.Velocity $ 
			"} {Visible " $ InFOV(IA.Location) $
			"} {Reachable " $ actorReachable(IA) $
			"}";
		myConnection.SendLine(outstring);
	}

	/*foreach DynamicActors(class'PickupFactory',Pickup)
	{
		if( (Pickup.GetStateName() == 'Pickup') && !Pickup.bHidden && CanSeeByPoints( Pawn.Location, Pickup.Location, Pawn.Rotation))//; inFOV(Pickup.Location) && LineOfSightTo(Pickup) )
		{
			myConnection.SendLine("INV {Id " $ myConnection.GetUniqueId(Pickup) $
				"} {Location " $ Pickup.Location $
				"} {Type " $ Pickup.Class $
				"}");
		}
	}*/
	foreach WorldInfo.AllNavigationPoints(class'NavigationPoint', N)
	{
		//Added attribute Visible, so inFOV is not here anymore
		if( FastTrace(N.Location, Pawn.Location) )//LineOfSightTo(N))//
		{
				outstring = "NAV {Id " $ N $
					"} {Location " $ N.Location $
					"} {Visible " $ inFOV( N.Location ) $
					"} {Reachable " $ actorReachable(N) $
					"}";
			/*	Location returned to all NavPoints for now
			if (N.IsA('LiftCenter'))
   				{
   					outstring = outstring $ " {Location "$ N.Location $"}";
   				}*/
				myConnection.SendLine(outstring);
			
		}
	}//end for N=WorldInfo.NavigationPointList
	
}


simulated event Destroyed()
{
	//Destroying bot Pawn
	DestroyPawn(); //kill him properly first
	if (Pawn != None) {
		Pawn.Destroy();//destroy him
		Pawn = None;
    }

	//Destroying actor for aiming our shooting on location
    if (myTarget != none) {
		myTarget.Destroy();
	}

	if (myConnection != none)
		myConnection.SendLine("FIN");
	else
		`log("Problem with sending FIN, myConnection = none");

    Super.Destroyed();
}

function ChangeWeapon();
state MoveToGoal
{
	function BeginState(Name PreviousStateName);

}

function ServerChangedWeapon(Weapon OldWeapon, Weapon NewWeapon);


//-----------------

defaultproperties
{
	remoteVisionLimit=0.707000
	SpeedMultiplier=1.0
	bDrawTraceLines=true
	bIncludeFadeOutInMsg=true
	bShowFocalPoint=false
	bPerfectLocationAim=false
	bSpeakingBots=true
	bAutoTrace=false
	bAutoSpawn=true
	MaxSpeed=2.00000
	DesiredSkin="ThunderCrash.JacobM"
	PawnClass=Class'GameBotsUDK.GBPawn'
}
